/*
 * libexplain - Explain errno values returned by libc functions
 * Copyright (C) 2009 Peter Miller
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include <libexplain/ac/net/if.h>

#include <libexplain/buffer/ifreq_ifindex.h>
#include <libexplain/buffer/pointer.h>
#include <libexplain/path_is_efault.h>


void
explain_buffer_ifreq_ifindex(explain_string_buffer_t *sb,
    const struct ifreq *data)
{
#if defined(ifr_ifindex)
    if (explain_pointer_is_efault(data, sizeof(*data)))
        explain_buffer_pointer(sb, data);
    else
    {
        const struct ifreq *ifr;

        /*
         * This is actually a huge big sucky union.  This specific
         * case gives the interface index.
         */
        ifr = data;
        explain_string_buffer_printf
        (
            sb,
            "{ ifr_ifindex = %d }",
            ifr->ifr_ifindex
        );
    }
#elif defined(ifr_index)
    if (explain_pointer_is_efault(data, sizeof(*data)))
        explain_buffer_pointer(sb, data);
    else
    {
        const struct ifreq *ifr;

        /*
         * This is actually a huge big sucky union.  This specific
         * case gives the interface index.
         */
        ifr = data;
        explain_string_buffer_printf
        (
            sb,
            "{ ifr_index = %d }",
            ifr->ifr_index
        );
    }
#else
    explain_buffer_pointer(sb, data);
#endif
}
